'use strict';

var ws = require('websocket-stream');

var url = require('url');

var makeRpcPlugin = require('./shared');

module.exports = {
  name: 'rpc-server',
  plugin: makeRpcPlugin(websocketServer)
};

function websocketServer(app, opts, handle) {
  opts = opts || {};
  var defaults = {
    perMessageDeflate: false,
    port: null,
    host: null,
    server: null,
    prefix: null
  };
  opts = Object.assign({}, defaults, opts);

  if (!opts.server && !opts.port && app.httpServer) {
    opts.server = app.httpServer;
  }

  var websocket = ws.createServer(opts, onConnection); // todo: error handling.

  websocket.on('error', function (err) {
    return console.log('socket server: error', err);
  });
  websocket.on('connection', function (socket) {
    socket.on('close', function (err) {
      return console.log('socket socket: client closed connection', err);
    });
    socket.on('error', function (err) {
      return console.log('socket socket: error', err);
    });
  });

  function onConnection(stream, req) {
    stream.on('error', function (err) {
      return console.log('ws stream: error', err);
    });

    if (opts.prefix) {
      var reqUrl = url.parse(req.url);
      if (reqUrl.pathname !== opts.prefix) return;
    }

    handle(stream, req);
  }
}
